import { Input } from "shader-composer";
import { ModuleFactory, ModulePipe } from ".";
export type BlendableType = "float" | "vec2" | "vec3" | "vec4";
export type BlendFunction = <T extends BlendableType>(a: Input<T>, b: Input<T>, opacity: Input<"float">) => Input<T>;
/**
 * Provides a numerb of different blend modes that can be used with a {@link Layer}.
 */
export declare const Blend: {
    normal: BlendFunction;
    discard: BlendFunction;
    add: BlendFunction;
};
export type BlendMode = keyof typeof Blend;
export type LayerArgs = {
    modules?: ModulePipe;
    /**
     * The opacity of the layer, between 0 and 1. Defaults to 1.
     */
    opacity?: Input<"float">;
    /**
     * The blend mode to use. Can be any function
     * that (@see {@link BlendMode}). Defaults to {@link Blend.normal}.
     */
    blend?: BlendFunction | BlendMode;
};
/**
 * Layers allow you to group any number of modules (see {@link Module}) and (optionally)
 * apply a blend mode and an opacity to them. Blending will have different effects on different
 * parts of the state; for example, the _blend mode_ will only apply to the color channel, while
 * position, normal and so on will simply be interpolated between according to the configured opacity.
 *
 * @param props - The arguments to the layer. See {@link LayerArgs}.
 */
export declare const Layer: ModuleFactory<LayerArgs>;
