import { pipe } from 'fp-ts/function';
import { $, Add, Vec3, Mul, Pow, Fresnel as Fresnel$1, VertexPosition, Smoothstep, Gradient as Gradient$1, Mat3, PSRDNoise3D, isUnit, UniformUnit, Texture2D, UV } from 'shader-composer';
import { _ as _taggedTemplateLiteral, L as Layer, a as _toConsumableArray } from '../../dist/index-a9db23fe.esm.js';
import { Billboard as Billboard$1, Softness as Softness$1 } from 'shader-composer-toybox';
import { Color as Color$1 } from 'three';
import '@material-composer/patch-material';

function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}

function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }
  return keys;
}
function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }
  return target;
}

var _templateObject$1, _templateObject2, _templateObject3;
/* TODO: extract into Shader Composer! */
var conditionalInstanceMatrix = $(_templateObject$1 || (_templateObject$1 = _taggedTemplateLiteral(["\n  #ifdef USE_INSTANCING\n  * instanceMatrix\n  #endif\n"])));
var convertToLocal = function convertToLocal(v, space) {
  return space === "world" ? Vec3($(_templateObject2 || (_templateObject2 = _taggedTemplateLiteral(["vec3(vec4(", ", 1.0) * modelMatrix ", ")"])), v, conditionalInstanceMatrix)) : space === "view" ? Vec3($(_templateObject3 || (_templateObject3 = _taggedTemplateLiteral(["vec3(vec4(", ", 1.0) * viewMatrix * modelMatrix ", ")"])), v, conditionalInstanceMatrix)) : v;
};

/**
 * Applies an offset to the vertex position. The user is expected to provide
 * the offset in the intended space; no further transformation is performed.
 */
var Translate = function Translate(_ref) {
  var offset = _ref.offset,
    _ref$space = _ref.space,
    space = _ref$space === void 0 ? "local" : _ref$space;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      position: pipe(offset, function (v) {
        return convertToLocal(v, space);
      }, function (v) {
        return Add(state.position, v);
      })
    });
  };
};

var Acceleration = function Acceleration(_ref) {
  var direction = _ref.direction,
    time = _ref.time,
    _ref$space = _ref.space,
    space = _ref$space === void 0 ? "world" : _ref$space;
  return Translate({
    space: space,
    offset: pipe(direction, function (v) {
      return Mul(v, Pow(time, 2));
    }, function (v) {
      return Mul(v, 0.5);
    })
  });
};

/**
 * Sets or modifies the fragment's alpha value.
 *
 * @param param0.alpha The alpha value to use. If a function is provided,
 * it will be called with the current alpha value.
 *
 * @example
 * This will set the alpha value to 0.5:
 *
 * ```jsx
 * Alpha({ alpha: 0.5 })
 * ```
 *
 * Just like any other module, you can use a Shader Composer sub-graph here:
 *
 * ```jsx
 * Alpha({ alpha: NormalizePlusMinusOne(Sin(Time())) })
 * ```
 *
 * Alternatively, you can pass a function; this function will receive the _current_ alpha value as its argument.
 * This gives you a chance to modify the value instead of overwriting it:
 *
 * ```jsx
 * Alpha({ alpha: (alpha) => Mul(alpha, 0.5) })
 * ```
 */
var Alpha = function Alpha(_ref) {
  var alpha = _ref.alpha;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      alpha: typeof alpha === "function" ? alpha(state.alpha) : alpha
    });
  };
};

var Billboard = function Billboard() {
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      position: Billboard$1(state.position)
    });
  };
};

/**
 * Sets or modifies the material's color.
 *
 * @param param0.color The color to use. Can be a `THREE.Color`
 * instance, a color representation compatible with `THREE.ColorRepresentation`,
 * or a function that receives the current color and returns a new color.
 *
 * @example
 * Set the color to red:
 *
 * ```jsx
 * Color({ color: 0xff0000 })
 * Color({ color: "red" })
 * Color({ color: new Color(1, 0, 0) })
 * ```
 *
 * Just like any other module, you can use a Shader Composer sub-graph here:
 *
 * ```jsx
 * Color({
 *  color: Mul(new Color("hotpink"), NormalizePlusMinusOne(Sin(Time())))
 * })
 * ```
 *
 * You can also provide a function; this function will receive the _current_ color as its argument,
 * and is expected to return a new color.
 *
 * ```jsx
 * Color({
 *   color: (color) => Mul(color, 5)
 * })
 * ```
 */
var Color = function Color(_ref) {
  var color = _ref.color;
  return function (state) {
    /* Determine new color */
    var newColor = /* If it's a color representation, create a new color object */
    typeof color === "string" || typeof color === "number" ? new Color$1(color) : /* If it's a function, call it with the previous color */
    typeof color === "function" ? color(state.color) : /* Otherwise, return the color value as-is */
    color;
    return _objectSpread2(_objectSpread2({}, state), {}, {
      color: newColor
    });
  };
};

var Fresnel = function Fresnel(props) {
  return Layer({
    opacity: Fresnel$1(props),
    modules: [Color({
      color: "white"
    })]
  });
};

var Gradient = function Gradient(_ref) {
  var stops = _ref.stops,
    _ref$contrast = _ref.contrast,
    contrast = _ref$contrast === void 0 ? 1 : _ref$contrast,
    _ref$start = _ref.start,
    start = _ref$start === void 0 ? 1 : _ref$start,
    _ref$stop = _ref.stop,
    stop = _ref$stop === void 0 ? -1 : _ref$stop,
    _ref$position = _ref.position,
    position = _ref$position === void 0 ? VertexPosition.y : _ref$position;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      color: pipe(position, function (v) {
        return Mul(v, contrast);
      }, function (v) {
        return Smoothstep(start, stop, v);
      }, function (v) {
        return Gradient$1.apply(void 0, [v].concat(_toConsumableArray(stops)));
      })
    });
  };
};

var _templateObject;
var Lifetime = function Lifetime(_ref) {
  var progress = _ref.progress;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      color: Vec3(state.color, {
        fragment: {
          body: $(_templateObject || (_templateObject = _taggedTemplateLiteral(["if (", " < 0.0 || ", " > 1.0) discard;"])), progress, progress)
        }
      })
    });
  };
};

var Rotate = function Rotate(_ref) {
  var rotation = _ref.rotation,
    _ref$normal = _ref.normal,
    normal = _ref$normal === void 0 ? true : _ref$normal;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      normal: normal ? Mul(state.normal, Mat3(rotation)) : state.normal,
      position: Mul(state.position, Mat3(rotation))
    });
  };
};

var Scale = function Scale(_ref) {
  var _ref$scale = _ref.scale,
    scale = _ref$scale === void 0 ? 1 : _ref$scale;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      position: Mul(state.position, scale)
    });
  };
};

var Softness = function Softness(_ref) {
  var softness = _ref.softness,
    depthTexture = _ref.depthTexture;
  return function (state) {
    return _objectSpread2(_objectSpread2({}, state), {}, {
      alpha: Mul(state.alpha, Softness$1(softness, state.position, depthTexture))
    });
  };
};

var SurfaceWobble = function SurfaceWobble(_ref) {
  var _ref$offset = _ref.offset,
    offset = _ref$offset === void 0 ? 1 : _ref$offset,
    _ref$amplitude = _ref.amplitude,
    amplitude = _ref$amplitude === void 0 ? 1 : _ref$amplitude,
    _ref$frequency = _ref.frequency,
    frequency = _ref$frequency === void 0 ? 1 : _ref$frequency;
  return function (state) {
    var displacement = pipe(state.position, function (v) {
      return Mul(v, frequency);
    }, function (v) {
      return Add(v, offset);
    }, function (v) {
      return PSRDNoise3D(v);
    }, function (v) {
      return Mul(v, amplitude);
    });
    var position = pipe(displacement, function (v) {
      return Mul(state.normal, v);
    }, function (v) {
      return Add(state.position, v);
    });
    return _objectSpread2(_objectSpread2({}, state), {}, {
      position: position
    });
  };
};

/**
 * Applies a texture to the material. The `texture` parameter can either be a
 * Shader Composer unit of type `sampler2D`, or a Three.js texture instance.
 * Note that when you pass the latter, it will automatically be converted into
 * a uniform.
 *
 * @param params0.texture A Three.js texture or a Shader Composer unit of type `sampler2D`.
 * @param params0.uv The UV coordinates to use for the texture. Defaults to the UV data provided by the material.
 */
var Texture = function Texture(_ref) {
  var texture = _ref.texture,
    _ref$uv = _ref.uv,
    uv = _ref$uv === void 0 ? UV : _ref$uv;
  return function (state) {
    var sampler = isUnit(texture) ? texture : UniformUnit("sampler2D", texture);
    var sampled = Texture2D(sampler, uv);
    return _objectSpread2(_objectSpread2({}, state), {}, {
      color: sampled.color,
      alpha: sampled.alpha
    });
  };
};

var Velocity = function Velocity(_ref) {
  var direction = _ref.direction,
    time = _ref.time,
    _ref$space = _ref.space,
    space = _ref$space === void 0 ? "world" : _ref$space;
  return Translate({
    offset: Mul(direction, time),
    space: space
  });
};

var CustomModule = function CustomModule(_ref) {
  var module = _ref.module;
  return module;
};

export { Acceleration, Alpha, Billboard, Color, CustomModule, Fresnel, Gradient, Lifetime, Rotate, Scale, Softness, SurfaceWobble, Texture, Translate, Velocity };
